@@ -193,7 +193,11 @@ class dyagram:
                 print("\nChanges in state!\n")
                 file = open(rf"{self.site}/state.json", 'r')
                 state = json.load(file)
-                diffs = self.get_state_diff(state, self.topology) # state is the state file on record and topology is current state
+                try:
+                    diffs = self.get_state_diff(state, self.topology) # state is the state file on record and topology is current state
+                except Exception as e:
+                    print("EXCEPTION")
+                    print(e)
                 # print to screen the diffs
                 print("DIFFS\n")
                 for ip in diffs:
@@ -405,28 +409,30 @@ class dyagram:
                             f"{routing_protocol} Neighbor {diff_resp[k][diff]['new_value']}")
 
                 elif category == "layer2":
-                    print("LAYER2 CHANGE DETECTED")
                     if diff_type != "changed":
                         device_diffs[s['devices'][device_elem]['inventory_ip']][diff_type].append(
                             f"LLDP Neighbor {diff_resp[k][diff]['hostname']} (Chassis ID: {diff_resp[k][diff]['chassis_id']})")
+
                     else:
-                        print("diff type == changed")
                         resp = re.search("\['neighbors'\]\[(\d+)\]\['(.*)'\]", diff)
                         neigh_elem = int(resp.group(1))
                         neigh_property = resp.group(2)
-                        print("before append")
                         device_diffs[s['devices'][device_elem]['inventory_ip']][diff_type].append(
                             f"LLDP Neighbor {s['devices'][device_elem][category]['neighbors'][neigh_elem]['hostname']}: {neigh_property}: "
                             f"NEW VALUE: {diff_resp[k][diff]['new_value']}, OLD VALUE: {diff_resp[k][diff]['old_value']}")
-                        print("after append")
                 elif category == "routes":
 
                     if diff_type != "changed":
 
-                        device_diffs[s['devices'][device_elem]['inventory_ip']][diff_type].append(
+                        try:
+                            device_diffs[s['devices'][device_elem]['inventory_ip']][diff_type].append(
 
-                            f"Route {diff_resp[k][diff]['network']}/{diff_resp[k][diff]['mask']} {diff_resp[k][diff]['next_hop']}")
+                                f"Route {diff_resp[k][diff]['network']}/{diff_resp[k][diff]['mask']} {diff_resp[k][diff]['next_hop']}")
+                        except KeyError:
+                            # different return for next hop for XR
+                            device_diffs[s['devices'][device_elem]['inventory_ip']][diff_type].append(
 
+                                f"Route {diff_resp[k][diff]['network']}/{diff_resp[k][diff]['mask']} {diff_resp[k][diff]['nexthop_ip']}")
                     else:
 
                         resp = re.search("\['routes'\]\[(\d+)\]\['(.*)'\]", diff)
@@ -436,20 +442,38 @@ class dyagram:
                         if route_elem not in elem_changed['routes']:
                             elem_changed['routes'].append(route_elem)
 
-                            device_diffs[s['devices'][device_elem]['inventory_ip']]["removed"].append(
-                                f"Route {s['devices'][device_elem][category][route_elem]['network']}/"
+                            try:
 
-                                f"{previous_state['devices'][device_elem][category][route_elem]['mask']} "
+                                device_diffs[s['devices'][device_elem]['inventory_ip']]["removed"].append(
+                                    f"Route {s['devices'][device_elem][category][route_elem]['network']}/"
+        
+                                    f"{previous_state['devices'][device_elem][category][route_elem]['mask']} "
+        
+                                    f"{previous_state['devices'][device_elem][category][route_elem]['next_hop']}")
 
-                                f"{previous_state['devices'][device_elem][category][route_elem]['next_hop']}")
+                                device_diffs[s['devices'][device_elem]['inventory_ip']]["added"].append(
 
-                            device_diffs[s['devices'][device_elem]['inventory_ip']]["added"].append(
+                                    f"Route {s['devices'][device_elem][category][route_elem]['network']}/"
+        
+                                    f"{current_state['devices'][device_elem][category][route_elem]['mask']} "
+        
+                                    f"{current_state['devices'][device_elem][category][route_elem]['next_hop']}")
+                            except KeyError:
+                                # different return for next hop for XR
+                                device_diffs[s['devices'][device_elem]['inventory_ip']]["removed"].append(
+                                    f"Route {s['devices'][device_elem][category][route_elem]['network']}/"
 
-                                f"Route {s['devices'][device_elem][category][route_elem]['network']}/"
+                                    f"{previous_state['devices'][device_elem][category][route_elem]['mask']} "
 
-                                f"{current_state['devices'][device_elem][category][route_elem]['mask']} "
+                                    f"{previous_state['devices'][device_elem][category][route_elem]['nexthop_ip']}")
 
-                                f"{current_state['devices'][device_elem][category][route_elem]['next_hop']}")
+                                device_diffs[s['devices'][device_elem]['inventory_ip']]["added"].append(
+
+                                    f"Route {s['devices'][device_elem][category][route_elem]['network']}/"
+
+                                    f"{current_state['devices'][device_elem][category][route_elem]['mask']} "
+
+                                    f"{current_state['devices'][device_elem][category][route_elem]['nexthop_ip']}")
 
 
         return device_diffs
@@ -951,7 +975,9 @@ class dyagram:
             else:
                 self.log.info(f"DEVICE: {netmiko_session.host} - SSH : Querying for hostname - START")
                 sh_run_output = netmiko_session.send_command("sh run | inc hostname")
+                print("BEFORE HOSTNAME")
                 hostname = re.search("hostname\s+(.*)", sh_run_output).group(1)
+                print(f"HOSTNAME {hostname}")
                 self.log.info(f"DEVICE: {netmiko_session.host} - SSH : Querying for hostname - SUCCESSFUL")
 
                 return hostname
